package com.winhc.bigdata.spark.jobs.chance

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging
import org.apache.spark.sql.functions.col
import org.apache.spark.sql.types.{MapType, StringType, StructField, StructType}
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/7/7 11:25
 * @Description: 筛选出数据的具体变更
 */
object ChangeExtract {

  //判断两个map在指定key上是否相等，如不等反回不相等字段
  def getDoubleDataMap(iterable: Iterable[Map[String, String]]): (Map[String, String], Map[String, String]) = {
    val map = iterable.map(m => (m("change_flag"), m)).toMap
    (map("0"), map("1"))
  }

  def getHandleClazz(tableName: String, equCols: Seq[String]): {def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], String, String, String, String, Map[String, String])} = {
    val clazz = s"com.winhc.bigdata.spark.jobs.chance.$tableName"
    val foo = Class.forName(clazz)
      .getConstructors.head.newInstance(equCols)
      .asInstanceOf[ {
      def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], String, String, String, String, Map[String, String])
    }]
    foo
  }


  case class ChangeExtractHandle(s: SparkSession,
                                 project: String, //表所在工程名
                                 tableName: String, //表名（不加前后辍）
                                 primaryKey: String, //此维度主键
                                 inc_ds: String, //需要计算的分区
                                 primaryFields: Seq[String] //主要字段，该字段任意一个不同 则认为发生变化
                                ) extends LoggingUtils with Logging {
    @(transient@getter) val spark: SparkSession = s


    val target_eci_change_extract = "ads_change_extract"

    def calc(): Unit = {
      val cols = primaryFields.filter(!_.equals(primaryKey)).seq

      val ds = inc_ds.replace("-", "")

      val intersectCols = getColumns(s"$project.ads_$tableName").toSet & getColumns(s"$project.inc_ads_$tableName").toSet

      val otherAllCols = intersectCols.filter(!primaryKey.equals(_)).toSeq
      val all_cols = primaryKey +: otherAllCols :+ "change_flag"

      val lastDs_ads_all = getLastPartitionsOrElse(s"$project.ads_$tableName", "0")

      val handle = getHandleClazz(tableName, cols)

      val update_time = BaseUtil.nowDate()

      val inc_ods_company = s"${project}.inc_ods_company" //每日公司基本信息增量
      val inc_ods_company_tb = s"${project}.inc_ods_$tableName" //增量ods表
      //增量ads最后一个分区
      val lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_$tableName", spark)

      val list = sql(s"show partitions $inc_ods_company_tb").collect.toList.map(_.getString(0).split("=")(1))
      //增量ods第一个分区
      val firstDsIncOds = list.head
      //增量ods最后一个分区//落表分区
      val lastDsIncOds = list.last
      //执行分区
      var runDs = ""
      //第一次run
      if (StringUtils.isBlank(lastDsIncAds)) {
        runDs = firstDsIncOds
      } else { //非第一次分区时间 + 1天
        runDs = BaseUtil.atDaysAfter(1, lastDsIncAds)
      }
      sql(
        s"""
           |SELECT  cid,current_cid as new_cid
           |FROM    ${inc_ods_company}
           |WHERE   ds >= ${runDs}
           |AND     cid IS NOT NULL
           |AND     current_cid IS NOT NULL
           |GROUP BY cid,current_cid
           |""".stripMargin).createOrReplaceTempView("mapping")

      val rdd = sql(
        s"""
           |SELECT  $primaryKey,${otherAllCols.mkString(",")},'0' as change_flag
           |FROM    $project.inc_ads_$tableName
           |WHERE   ds = $ds
           |UNION ALL
           |SELECT  t2.$primaryKey,${otherAllCols.map("t2." + _).mkString(",")},'1' as change_flag
           |FROM    (
           |            SELECT  DISTINCT ${primaryKey}
           |            FROM    $project.inc_ads_$tableName
           |            WHERE   ds = $ds
           |        ) AS t1
           |JOIN    (
           |             SELECT  concat_ws('_',coalesce(mm.new_cid,tmp.cid),split(rowkey, '_')[1]) AS rowkey
           |                     ,${intersectCols.filter(s => {!s.equals("rowkey") && !s.equals("cid") && !s.equals("new_cid")}).mkString(",")}
           |                     ,coalesce(mm.new_cid,tmp.cid) AS new_cid
           |                     ,tmp.cid
           |                     ,c
           |             FROM    (
           |                         SELECT  a.*
           |                                 ,row_number() OVER (PARTITION BY a.${primaryKey} ORDER BY update_time DESC) c
           |                         FROM    (
           |                                     SELECT  ${intersectCols.mkString(",")}
           |                                     FROM    $project.ads_$tableName
           |                                     WHERE   ds = $lastDs_ads_all
           |                                     UNION ALL
           |                                     SELECT  ${intersectCols.mkString(",")}
           |                                     FROM    $project.inc_ads_$tableName
           |                                     WHERE   ds > $lastDs_ads_all and ds < $ds
           |                                 ) AS a
           |                     ) AS tmp
           |             LEFT JOIN mapping mm
           |             ON tmp.cid = mm.cid
           |             WHERE   tmp.c = 1
           |        ) AS t2
           |ON      t1.${primaryKey} = t2.${primaryKey}
           |""".stripMargin)
        .select(all_cols.map(column => col(column).cast("string")): _*)
        .rdd.map(r => {
        (r.getAs[String](primaryKey), all_cols.map(f => (f, r.getAs[String](f))).toMap)
      }).groupByKey()
        .map(x => {
          val rowkey = x._1
          val map_list = x._2
          //          try {
          if (map_list.size == 1) {
            val res = handle.handle(rowkey, null, map_list.head)
            Row(res._1, res._2, tableName, res._3, res._4, res._5, res._6, res._7, res._8, update_time, res._9)
          } else {
            if (map_list.size > 2) {
              logger.error("list.size greater than 2! rowkey:" + rowkey)
            }
            val m = getDoubleDataMap(map_list)

            val new_map = m._1
            val old_map = m._2
            val res = handle.handle(rowkey, old_map, new_map)
            if (res == null) {
              null
            } else {
              Row(res._1, res._2, tableName, res._3, res._4, res._5, res._6, res._7, res._8, update_time, res._9)
            }
          }
          /* } catch {
             case e: Exception => {
               logError(s"xjk rowkey:$rowkey msg:${e.getMessage} equCols:$cols")
               logError(e.getMessage, e)
               println(s"xjk rowkey:$rowkey msg:${e.getMessage} equCols:$cols")
             }
               null
           }*/
        }).filter(_ != null)

      // (123_abc,insert,{a->b},all,新增某土地公示，1（1.一般变更，2.风险变更）)
      val schema = StructType(Array(
        StructField("rowkey", StringType), //表数据主建
        StructField("cid", StringType), //公司id
        StructField("table_name", StringType), //表名
        StructField("type", StringType), // 变更类型 insert update
        StructField("data", MapType(StringType, StringType)), //变更后数据
        StructField("fields", StringType), //如果是更新 则显示更新字段
        StructField("title", StringType), // 动态数据展示 ps. 新增某土地公示
        StructField("label", StringType), // 1.一般变更，2.风险变更
        StructField("biz_time", StringType), //业务时间
        StructField("update_time", StringType), //处理时间
        StructField("old_data", MapType(StringType, StringType)) //变更前数据
      ))

      spark.createDataFrame(rdd, schema)
        .createOrReplaceTempView("tmp_change_extract_view") //

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${project}.$target_eci_change_extract PARTITION(ds='$ds',tn='$tableName')
           |SELECT *
           |FROM
           |    tmp_change_extract_view
           |""".stripMargin)
    }
  }


  // winhc_eci_dev company_tm rowkey 20200717 status_new
  // winhc_eci_dev company_patent_list rowkey 20200717 lprs
  // winhc_eci_dev company_certificate rowkey 20200707 type
  // winhc_eci_dev company_copyright_works_list rowkey 20200717 type
  // winhc_eci_dev company_copyright_reg_list rowkey 20200717 version
  // winhc_eci_dev company_employment rowkey 20200630 source

  // winhc_eci_dev company_land_publicity rowkey 20200717 title,location,use_for
  // winhc_eci_dev company_land_announcement rowkey 20200717 e_number,project_name

  // winhc_eci_dev company_bid_list rowkey 20200717 title
  // winhc_eci_dev company_land_transfer rowkey 20200717 num,location
  // winhc_eci_dev company_abnormal_info rowkey 20200717 remove_reason


  // winhc_eci_dev company cid 20200630 legal_entity_id,reg_location,business_scope,reg_status,reg_capital,emails,phones
  def main(args: Array[String]): Unit = {
    if (args.length == 5) {
      val Array(project, tableName, rowkey, inc_ds, pf) = args
      val config = EsConfig.getEsConfigMap ++ mutable.Map(
        "spark.hadoop.odps.project.name" -> project,
        "spark.hadoop.odps.spark.local.partition.amt" -> "10"
      )
      val spark = SparkUtils.InitEnv("ChangeExtract", config)


      ChangeExtractHandle(spark, project, tableName, rowkey, inc_ds, pf.split(",")).calc
      spark.stop()
    } else {
      val ds = args(0)
      val project = "winhc_eci_dev"
      val config = EsConfig.getEsConfigMap ++ mutable.Map(
        "spark.hadoop.odps.project.name" -> project,
        "spark.hadoop.odps.spark.local.partition.amt" -> "10"
      )
      val spark = SparkUtils.InitEnv("ChangeExtract", config)
      val rows =
        """winhc_eci_dev company_tm rowkey 20200717 status_new
          |winhc_eci_dev company_patent_list rowkey 20200717 lprs
          |winhc_eci_dev company_copyright_works_list rowkey 20200717 type
          |winhc_eci_dev company_copyright_reg_list rowkey 20200717 version
          |winhc_eci_dev company_land_publicity rowkey 20200717 title,location,use_for
          |winhc_eci_dev company_land_announcement rowkey 20200717 e_number,project_name
          |winhc_eci_dev company_bid_list rowkey 20200717 title
          |winhc_eci_dev company_land_transfer rowkey 20200717 num,location
          |winhc_eci_dev company_employment rowkey 20200717 source
          |winhc_eci_dev company_env_punishment rowkey 20200717 punish_number
          |""".stripMargin.replace("20200717", ds)
      for (r <- rows.split("\r\n")) {
        if (StringUtils.isNotEmpty(r)) {
          val Array(tmp, tableName, rowkey, inc_ds, pf) = r.split(" ")
          ChangeExtractHandle(spark, project, tableName, rowkey, inc_ds, pf.split(",")).calc
        }
      }
      spark.stop()
    }
  }
}
