package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, MaxComputer2Phoenix, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable


/**
 * @Author: XuJiakai
 * @Date: 2020/7/29 11:18
 * @Description: 股权出质增量数据外理
 */
object inc_company_equity_info {


  case class IncCompanyEquityInfoUtils(s: SparkSession,
                                       project: String, //表所在工程名
                                       ds: String //此维度主键
                                      ) extends LoggingUtils with Logging with BaseFunc{
    @(transient@getter) val spark: SparkSession = s


    def calc(): Unit = {
      cleanup()
      val cols = getColumns(s"winhc_eci_dev.ads_company_equity_info").diff(Seq("ds", "rowkey", "id"))

//      val startPart = getLastPartitionsOrElse(s"winhc_eci_dev.inc_ads_company_equity_info", "0")
      val startPart = "20200725"
      val endPart = getLastPartitionsOrElse(s"winhc_eci_dev.inc_ods_company_equity_info", BaseUtil.getYesterday())

      if (startPart.equals(endPart)) {
        println("start partition = end partition!")
        sys.exit(-1)
      }

      sql(
        s"""
           |SELECT  tmp.id
           |        ,tmp.cid
           |        ,tmp.base
           |        ,tmp.reg_number
           |        ,tmp.pledgor
           |        ,tmp.certif_number_l
           |        ,tmp.equity_amount
           |        ,tmp.pledgee
           |        ,tmp.certif_number_r
           |        ,tmp.reg_date
           |        ,tmp.state
           |        ,tmp.pub_date
           |        ,tmp.change_situation
           |        ,tmp.cancel_date
           |        ,tmp.cancel_reason
           |        ,tmp.pledgor_type
           |        ,tmp.pledgor_id
           |        ,tmp.pledgee_type
           |        ,tmp.pledgee_id
           |        ,tmp.create_time
           |        ,tmp.update_time
           |        ,tmp.deleted
           |FROM    (
           |            SELECT  a.*
           |                    ,row_number() OVER (PARTITION BY a.reg_number,a.cid,a.pledgor,a.pledgee ORDER BY update_time DESC) c
           |            FROM    (
           |                        SELECT  *
           |                        FROM    winhc_eci_dev.inc_ods_company_equity_info
           |                        WHERE   ds > $startPart
           |                    ) as a
           |        ) AS tmp
           |WHERE   tmp.c = 1
           |""".stripMargin).createOrReplaceTempView("tmp_company_equity_info_all")


      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.inc_ads_company_equity_info PARTITION(ds='$endPart')
           |SELECT  md5(cleanup(CONCAT_WS('',reg_number,cid,pledgor,pledgee))) as id
           |        ,${cols.mkString(",")}
           |FROM    tmp_company_equity_info_all
           |""".stripMargin)

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci_dev.inc_ads_company_equity_info_list PARTITION(ds='$endPart')
           |SELECT  CONCAT_WS('_',pledgee_id,id) as rowkey
           |        ,pledgee_id AS cid
           |        ,id AS main_id
           |        ,pledgee AS cname
           |        ,2 AS type
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   pledgee_id IS NOT NULL
           |AND     pledgee_type == 2
           |
           |UNION ALL
           |
           |SELECT  CONCAT_WS('_',pledgor_id,id) as rowkey
           |        ,pledgor_id AS cid
           |        ,id AS main_id
           |        ,pledgor AS cname
           |        ,1 AS type
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   pledgor_id IS NOT NULL
           |AND     pledgor_type == 2
           |
           |UNION ALL
           |
           |SELECT  CONCAT_WS('_',cid,id) as rowkey
           |        ,cid AS cid
           |        ,id AS main_id
           |        ,null AS cname
           |        ,0 AS type
           |        ,state
           |        ,deleted
           |FROM    tmp_company_equity_info_all
           |WHERE   cid IS NOT NULL
           |""".stripMargin)

      val writeCols = getColumns("winhc_eci_dev.inc_ads_company_equity_info_list").diff(Seq("ds", "rowkey"))

      MaxComputer2Phoenix(spark
        , writeCols
        , "inc_ads_company_equity_info_list"
        , "COMPANY_EQUITY_INFO_LIST"
        , endPart
        , "CONCAT_WS('_',cid,main_id)").syn()
      import com.winhc.bigdata.spark.implicits.DataFrame2HBaseHelper._

      val outFields = getColumns("winhc_eci_dev.inc_ads_company_equity_info").map(_.toUpperCase)
      sql(
        s"""
           |SELECT  *
           |FROM    winhc_eci_dev.inc_ads_company_equity_info
           |WHERE   ds = '$endPart'
           |""".stripMargin)
        .save2HBase("COMPANY_EQUITY_INFO", "id", outFields)
    }
  }


  def main(args: Array[String]): Unit = {
    val Array(project, ds) = args

    println(
      s"""
         |project: $project
         |ds: $ds
         |""".stripMargin)

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv("inc_company_equity_info", config)
    IncCompanyEquityInfoUtils(spark, project, ds).calc()
    spark.stop()
  }

}
