package com.winhc.bigdata.spark.utils

import java.util.Date

import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 拆平cids,落表
 */

case class CompanyForCidsUtils(s: SparkSession, sourceTable: String, cols: Seq[String]) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._
  import spark._
  import org.apache.spark.sql.functions._

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)
    val odsTable = s"ods_$sourceTable"
    val adsListTable = s"ads_${sourceTable}_list"
    val adsTable = s"ads_$sourceTable"
    val companyMapping = "company_name_mapping_pro"
    val ds = BaseUtil.getPartion(odsTable, spark)
    //table字段
    val columns: Seq[String] = spark.table(odsTable).schema.map(_.name).filter(!_.equals("ds"))
    val disCol = cols

    sql(s"select * from $odsTable where ds = $ds and cids is not null and trim(cids) <> '' ")
      .dropDuplicates(disCol)
      .createOrReplaceTempView("t1")

    sql(s"CACHE TABLE t1")

    //拆平新表
    sql(
      s"""
         |SELECT
         |        rowkey,new_cid,${columns.mkString(",")}
         |FROM    (
         |        SELECT
         |                *
         |                ,ROW_NUMBER() OVER (PARTITION BY id,new_cid ORDER BY - ABS(CAST(new_cid AS BIGINT )- CAST(cid AS BIGINT )) DESC ) num
         |                ,CONCAT_WS('_',new_cid,id) AS rowkey
         |        FROM    (
         |                SELECT
         |                        c.*
         |                        ,coalesce(d.new_cid,c.cid) AS new_cid
         |                FROM    (
         |                        SELECT
         |                                *
         |                        FROM    t1 a
         |                        LATERAL VIEW explode(split(cids,';')) b AS cid
         |                        ) c
         |                LEFT JOIN $companyMapping d
         |                ON      c.cid = d.cid
         |                ) e
         |        ) f
         |WHERE   num =1
         |""".stripMargin)
      .createOrReplaceTempView(s"t2")

    //聚合新cids
    val df1 = sql(
      s"""
         |SELECT
         |x.new_cids,${columns.mkString(",")}
         |FROM    t1
         |LEFT JOIN (
         |              SELECT  id as new_id
         |                      ,concat_ws(';',collect_set(new_cid)) new_cids
         |              FROM    t2
         |              GROUP BY id
         |          ) x
         |ON      t1.id = x.new_id
         |""".stripMargin)

    df1.createOrReplaceTempView("t3")

    sql(s"select rowkey,new_cid,${columns.mkString(",")} from t2").show(10)
    sql(s"select new_cids,${columns.mkString(",")} from t3").show(10)

    //写表
    sql(s"insert overwrite table ${adsListTable} partition (ds=${ds}) select rowkey,new_cid,${columns.mkString(",")} from t2")
    sql(s"insert overwrite table ${adsTable} partition (ds=${ds}) select new_cids,${columns.mkString(",")} from t3")
    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}